 /*
  * sigact.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdbool.h>
#include <inttypes.h>
#include <errno.h>
#include <assert.h>
#include <sys/wait.h>
#include <dirent.h>
#include <signal.h>

#include <simple-netaid/sbuf.h>
#include <simple-netaid/ethlink.h>
#include <simple-netaid/netproc.h>
#include <simple-netaid/iproute.h>
#include <simple-netaid/interfaces.h>

#include "sigact.h"
#include "periodic_reminder.h"
#include <globals.h>

static int count = 1;
static volatile int active_wifis_interval = 1;
static volatile sig_atomic_t keep_going = 1;

static
void hdl (int signum, siginfo_t *info, void *extra)
{
   switch (signum)
   {
      case SIGALRM:
		 if (keep_going)
         {
			 sbuf_t s;
			 bool connected_ok = false;             
             const char * ifname = iproute();
			 if (*ifname != 0) connected_ok = true;
			 ui_update_menubar (connected_ok);
			 if (connected_ok) 
			    ui_update_toolbar (false);
			 else 
			    ui_update_toolbar (true);
			 
			 ui_update_statusbar (ifname);
			
             sbuf_init(&s);
             netproc(&s);
			 ui_update_status_icon (s.buf);
			 sbuf_free(&s);
		     keep_going = 0;
	     }
	     if (active_wifis_interval>0) {
			 if (count % active_wifis_interval == 0) {
	             ui_update();
			 }    
			 count++;    
		 }	     
         break;
      case SIGCHLD:
         break;
      case SIGHUP:
         break;
      case SIGUSR1:
         { 
			 int rc_int_val = info->si_value.sival_int;
			 if (rc_int_val == 1)
			     ui_wired_connection();
         }
         break;
      case SIGUSR2:
         {
             keep_going = 1;
			 int rc_int_val = info->si_value.sival_int;
			 if (rc_int_val == 2) {
				 ui_wireless_on ();
			 }
			 else if (rc_int_val == 3) {
				 ui_wired_connection ();
			 }
			 else if (rc_int_val < 0 || rc_int_val > 3)
			     active_wifis_interval = rc_int_val;
         }
         break;
      case SIGTERM:
      case SIGINT:
      case SIGQUIT:
         stop_periodic (); 
         /* Make sure to cleanup if user sends a keyboard interrupt */
         ui_delete ();
         break;
   }
}

void sigaction_init ()
{
   struct sigaction sa;
   memset (&sa, 0, sizeof(sa));
      
   /*---------------- initialize signal handling ------------ */
   sa.sa_sigaction = &hdl;
   sa.sa_flags = SA_RESTART | SA_NODEFER | SA_SIGINFO; //  SA_RESETHAND | 
   sigemptyset (&sa.sa_mask);
   sigaction (SIGALRM, &sa, 0);
   sigaction (SIGCHLD, &sa, 0);
   sigaction (SIGHUP,  &sa, 0);
   sigaction (SIGUSR1, &sa, 0);
   sigaction (SIGUSR2, &sa, 0);
   sigaction (SIGTERM, &sa, 0);
   sigaction (SIGINT,  &sa, 0);
   sigaction (SIGQUIT, &sa, 0);
}
